#include "showsort.h"
#include <stdio.h>

GtkWidget *drawingarea;

GdkGC *gc[BAR_COLORS];
GdkColormap *colormap;
GtkWidget *progress_bar;

void update_progress(gfloat percentage)
{
  gtk_progress_bar_update(GTK_PROGRESS_BAR(progress_bar), percentage);
}

void set_colors()
{
  int i;
  GdkColor color[BAR_COLORS];

  colormap = gdk_window_get_colormap(drawingarea->window);

  /* replace these indices with constant names */
  color[0].red = 5000;
  color[0].green = 5000;
  color[0].blue = 5000;

  color[1].red = 30000;
  color[1].green = 30000;
  color[1].blue = 30000;

  color[2].red = 5000;
  color[2].green = 65535;
  color[2].blue = 5000;

  color[3].red = 5000;
  color[3].green = 40000;
  color[3].blue = 5000;

  color[4].red = 65535;
  color[4].green = 5000;
  color[4].blue = 5000;

  color[5].red = 5000;
  color[5].green = 5000;
  color[5].blue = 65535;

  for (i = 0; i < BAR_COLORS; i++){
    color[i].pixel = (gulong)(color[i].red * 65536 + color[i].green * 256 + color[i].red);
    gdk_color_alloc(colormap, &color[i]);
    gc[i] = gdk_gc_new(drawingarea->window);
    gdk_gc_set_foreground(gc[i], &color[i]);
  }

}

void main_window()
{  
  GtkWidget *window;
  GtkWidget *mainvbox;
  GtkWidget *hbox1, *vbox, *box;
  GtkWidget *button;
  GSList *group;
  GtkWidget *separator;

  GtkWidget *toolbar;

  GtkWidget *menu_bar;

  /* main window */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "ShowSort");
  gtk_container_set_border_width(GTK_CONTAINER(window), 2);
  gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);

  gtk_signal_connect(GTK_OBJECT (window), "delete_event",
                     GTK_SIGNAL_FUNC (delete_event), NULL);

  gtk_signal_connect(GTK_OBJECT (window), "destroy",
                     GTK_SIGNAL_FUNC(destroy), NULL);

  gtk_widget_realize(window);

  /* vertical items */
  mainvbox = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(window), mainvbox);
  gtk_widget_show(mainvbox);

  /* menu bar */
  menu_bar = create_menu_bar(window);
  gtk_box_pack_start(GTK_BOX(mainvbox), menu_bar, FALSE, FALSE, 0);
  gtk_widget_show(menu_bar);
  
  /* toolbar etc. */
  toolbar = create_tool_bar(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(mainvbox), toolbar, FALSE, FALSE, 0);
  gtk_widget_show(toolbar);

  /* separator */
  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(mainvbox), separator, FALSE, FALSE, 2);
  gtk_widget_show(separator);

  /* drawing area */
  drawingarea = gtk_drawing_area_new();  
  gtk_drawing_area_size(GTK_DRAWING_AREA(drawingarea), 600, 200);
  gtk_signal_connect(GTK_OBJECT(drawingarea), "expose_event",
                     GTK_SIGNAL_FUNC(expose_event), NULL);
  gtk_signal_connect(GTK_OBJECT(drawingarea),"configure_event",
                     GTK_SIGNAL_FUNC(configure_event), NULL);
  
  gtk_box_pack_start(GTK_BOX(mainvbox), drawingarea, FALSE, FALSE, 0);
  gtk_widget_show(drawingarea);

  /* separator */
  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(mainvbox), separator, FALSE, FALSE, 2);
  gtk_widget_show(separator);

  /* progress bar */
  progress_bar = gtk_progress_bar_new();
  gtk_box_pack_start(GTK_BOX(mainvbox), progress_bar, FALSE, FALSE, 0);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(progress_bar), 0.0);
  gtk_widget_show(progress_bar);

  /* radio buttons */
  hbox1 = gtk_hbox_new(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(mainvbox), hbox1, FALSE, FALSE, 0);
  gtk_widget_show(hbox1);

  /* radio button set 1 */
  vbox = gtk_vbox_new(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(hbox1), vbox, FALSE, FALSE, 0);
  gtk_widget_show(vbox);

  /* forward button */
  button = gtk_radio_button_new_with_label(NULL, "Forward");
  gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0); 
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(set_direction), (gpointer)"forward");
  gtk_widget_show(button);

  /* reverse button */
  group = gtk_radio_button_group(GTK_RADIO_BUTTON(button));
  button = gtk_radio_button_new_with_label(group, "Reverse");
  gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0); 
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(set_direction), (gpointer)"reverse");
  gtk_widget_show(button);

  /* bi-directional button */
  group = gtk_radio_button_group(GTK_RADIO_BUTTON(button));
  button = gtk_radio_button_new_with_label(group, "Bi-directional");
  gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0); 
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(set_direction), (gpointer)"bi_directional");
  gtk_widget_show(button);

  /* radio button set 2 */
  vbox = gtk_vbox_new(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(hbox1), vbox, FALSE, FALSE, 0);
  gtk_widget_show(vbox);

  /* one by one button */
  button = gtk_radio_button_new_with_label(NULL, "One by one");
  gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0); 
  gtk_widget_show(button);

  /* one by one with flag button */
  group = gtk_radio_button_group(GTK_RADIO_BUTTON(button));
  button = gtk_radio_button_new_with_label(group, "One by one (flag)");
  gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0); 
  gtk_widget_show(button);

  /* remove sorted button */
  group = gtk_radio_button_group(GTK_RADIO_BUTTON(button));
  button = gtk_radio_button_new_with_label(group, "Remove sorted");
  gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0); 
  gtk_widget_show(button);

  /* button bar at the bottom */
  hbox1 = gtk_hbox_new(FALSE, 4);
  gtk_box_pack_start(GTK_BOX(mainvbox), hbox1, FALSE, FALSE, 0);
  gtk_widget_show(hbox1);

  /* reset button */
  button = gtk_button_new_with_label(" Reset ");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(reset_list), drawingarea);
  gtk_box_pack_start(GTK_BOX(hbox1), button, FALSE, FALSE, 0);
  gtk_widget_show(button);

  /* play button */
  button = gtk_button_new_with_label(" Start ");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(start_play), drawingarea);
  gtk_box_pack_start(GTK_BOX(hbox1), button, FALSE, FALSE, 0);
  gtk_widget_show(button);

  /* step button */
  button = gtk_button_new_with_label(" Step ");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(step_play), drawingarea);
  gtk_box_pack_start(GTK_BOX(hbox1), button, FALSE, FALSE, 0);
  gtk_widget_show(button);

  /* exit button */
  button = gtk_button_new_with_label(" Exit ");
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                            GTK_SIGNAL_FUNC(gtk_widget_destroy),
			     GTK_OBJECT(window));
  gtk_box_pack_start(GTK_BOX(hbox1), button, FALSE, FALSE, 0);

  gtk_widget_show(button);
  gtk_widget_show(window);


  set_colors();

}

