#include "showsort.h"

#define OFFSET 10

GdkPixmap *pixmap;
GtkWidget *drawingarea;

extern GtkWidget *sort_option_menu;
extern GtkWidget *data_option_menu;

extern int n;
extern int x[MAXITEMS];
extern int col[MAXITEMS];
extern int working;
extern int sort_type;
extern int list_type;
extern int list_used;
extern int sort_direction;
extern int ready_to_leave;

extern GdkGC *gc[BAR_COLORS];
extern GdkColormap *colormap;

state_type current;

void init;

int stepping = FALSE;

/* Create a new backing pixmap of the appropriate size */
gint configure_event(GtkWidget *widget, GdkEventConfigure *event)
{
  if (pixmap) gdk_pixmap_unref(pixmap);

  pixmap = gdk_pixmap_new(widget->window,
                          widget->allocation.width,
                          widget->allocation.height, -1);

 gdk_draw_rectangle(pixmap,
                    widget->style->white_gc,
                    TRUE, 0, 0,
                    widget->allocation.width,
		    widget->allocation.height);

   return TRUE;
}


/* Redraw the screen from the backing pixmap */
gint expose_event(GtkWidget *widget, GdkEventExpose *event)
{
  gdk_draw_pixmap(widget->window,
                  widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
	          pixmap, event->area.x, event->area.y,
                  event->area.x, event->area.y,
                  event->area.width, event->area.height);

    return FALSE;
}


gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
#ifdef DEBUG
  g_print("delete event occurred\n");
#endif
  return(FALSE);
}

void destroy(GtkWidget *widget, gpointer data)
{
  if (working) {
    working = FALSE;
  }
  while (gtk_events_pending())
    gtk_main_iteration();

#ifdef DEBUG
  g_print("destroy event\n");
#endif

  gtk_main_quit();
}

void update_pic(int n1, int n2)
{
  int i;
  GdkRectangle update_rect;

  for (i = n1; i <= n2; i++){
    update_rect.x = i * 5 + OFFSET;
    update_rect.y = 10;
    update_rect.width = 4;
    update_rect.height = 190;
    gdk_draw_rectangle(pixmap, drawingarea->style->white_gc, TRUE,
                   update_rect.x, update_rect.y,
                   update_rect.width, update_rect.height);

    update_rect.x = i * 5 + OFFSET;
    update_rect.y = 190-x[i];
    update_rect.width = 4;
    update_rect.height = x[i];
    gdk_draw_rectangle(pixmap, gc[col[i]], TRUE,
                   update_rect.x, update_rect.y,
      	           update_rect.width, update_rect.height);

    update_rect.y = 10;
    update_rect.height = 190;
    gtk_widget_draw(drawingarea, &update_rect);
  }

  usleep(1000);

  while (gtk_events_pending())
    gtk_main_iteration();
}

void reset_list(GtkWidget *widget, gpointer data)
{
  int i;

  if (working || (list_used == FALSE)) return;

  working = TRUE;

  srand((unsigned long)time(NULL));

  switch (list_type) {
    case SORTED:
    case ALMOST_SORTED:
      for (i = 0; i < n; i++){
        x[i] = DATAMIN + (DATAMAX - DATAMIN) * i / n;
      }
      break;
    case REVERSE:
    case ALMOST_REVERSE:
      for (i = 0; i < n; i++)
        x[i] = DATAMAX - (DATAMAX - DATAMIN) * i / n;
      break;
    case RANDOM:
      for (i = 0; i < n; i++)
	x[i] = rand() % (DATAMAX - DATAMIN) + DATAMIN;
      break;
  }

  for (i = 0; i < n; i++)
    col[i] = COLOR_NORMAL;

  current.state = STATE_BEGIN;

  if (sort_direction == SORT_BI_DIRECTIONAL)
    current.direction = SORT_FORWARD;
  else
    current.direction = sort_direction;
  
  list_used = FALSE;
  update_progress(0.0);

  update_pic(0, n - 1);

  working = FALSE;
}

void quick_sort()
{

}

void insertion_sort()
{

}

void selection_sort()
{

}

void step_play(GtkWidget *widget, gpointer data)
{
  if (working) return;
  stepping = TRUE;
  start_play(widget, data);
  stepping = FALSE;
}

void start_play(GtkWidget *widget, gpointer data)
{

  if (working){
    working = FALSE;
    return;
  }

  working = TRUE;
  ready_to_leave = FALSE;
  gtk_widget_set_sensitive(sort_option_menu, FALSE);
  gtk_widget_set_sensitive(data_option_menu, FALSE);

  switch (sort_type){
    case BUBBLE_SORT :
      if (stepping)
	bubble_sort_step();
      else
        while ((bubble_sort_step() == FALSE) && working);
      break;
    case INSERTION_SORT :
      insertion_sort();
      break;
    case SELECTION_SORT :
      selection_sort();
      break;
    case QUICK_SORT :
      quick_sort();
      break;
  }

  gtk_widget_set_sensitive(sort_option_menu, TRUE);
  gtk_widget_set_sensitive(data_option_menu, TRUE);
  working = FALSE;
  ready_to_leave = TRUE;

}

void set_direction (GtkWidget *widget, gpointer data)
{
  if (strcmp((char*)data, "forward") == 0){
    sort_direction = SORT_FORWARD;
  }
  else if (strcmp((char*)data, "reverse") == 0){
    sort_direction = SORT_REVERSE;
  }
  else if (strcmp((char*)data, "bi_directional") == 0){
    sort_direction = SORT_BI_DIRECTIONAL;
  }
}
