#include "showsort.h"

GtkWidget *sort_option_menu;
GtkWidget *data_option_menu;

extern int sort_type;
extern int list_type;
extern int list_used;

void set_sort_type(gchar *string);
void set_data_type(gchar *string);

GtkWidget *create_tool_bar()
{  
  GtkWidget *quick_item, *bubble_item, *selection_item, *insertion_item;
  GtkWidget *random_item, *sorted_item, *reverse_item, *a_sorted_item, *a_reverse_item;
  GtkWidget *menu;
  GtkWidget *toolbar;

  /* toolbar etc. */
  toolbar = gtk_hbox_new(FALSE, 2);

  /* sort type - option menu */
  sort_option_menu = gtk_option_menu_new();
  menu = gtk_menu_new();
  bubble_item = gtk_menu_item_new_with_label("Bubble sort");
  selection_item = gtk_menu_item_new_with_label("Selection sort");
  insertion_item = gtk_menu_item_new_with_label("Insertion sort");
  quick_item = gtk_menu_item_new_with_label("Quick sort");

  gtk_menu_append(GTK_MENU(menu), bubble_item);
  gtk_menu_append(GTK_MENU(menu), selection_item);
  gtk_menu_append(GTK_MENU(menu), insertion_item);
  gtk_menu_append(GTK_MENU(menu), quick_item);

  gtk_widget_show(bubble_item);
  gtk_widget_show(selection_item);
  gtk_widget_show(insertion_item);
  gtk_widget_show(quick_item);

  gtk_signal_connect_object(GTK_OBJECT(bubble_item), "activate",
		            GTK_SIGNAL_FUNC(set_sort_type),
			    (gpointer)"bubble");
  gtk_signal_connect_object(GTK_OBJECT(selection_item), "activate",
		            GTK_SIGNAL_FUNC(set_sort_type),
			    (gpointer)"selection");
  gtk_signal_connect_object(GTK_OBJECT(insertion_item), "activate",
		            GTK_SIGNAL_FUNC(set_sort_type),
			    (gpointer)"insertion");
  gtk_signal_connect_object(GTK_OBJECT(quick_item), "activate",
		            GTK_SIGNAL_FUNC(set_sort_type),
			    (gpointer)"quick");

  gtk_widget_show(menu);

  gtk_option_menu_set_menu(GTK_OPTION_MENU(sort_option_menu), menu);
  gtk_box_pack_start(GTK_BOX(toolbar), sort_option_menu, FALSE, FALSE, 0);
  gtk_widget_show(sort_option_menu);

  /* initial data type - option menu */
  data_option_menu = gtk_option_menu_new();
  menu = gtk_menu_new();
  random_item = gtk_menu_item_new_with_label("Random");
  sorted_item = gtk_menu_item_new_with_label("Sorted");
  reverse_item = gtk_menu_item_new_with_label("Reverse");
  a_sorted_item = gtk_menu_item_new_with_label("Almost sorted");
  a_reverse_item = gtk_menu_item_new_with_label("Almost reverse");

  gtk_menu_append(GTK_MENU(menu), random_item);
  gtk_menu_append(GTK_MENU(menu), sorted_item);
  gtk_menu_append(GTK_MENU(menu), reverse_item);
  gtk_menu_append(GTK_MENU(menu), a_sorted_item);
  gtk_menu_append(GTK_MENU(menu), a_reverse_item);

  gtk_widget_show(random_item);
  gtk_widget_show(sorted_item);
  gtk_widget_show(reverse_item);
  gtk_widget_show(a_sorted_item);
  gtk_widget_show(a_reverse_item);

  gtk_signal_connect_object(GTK_OBJECT(random_item), "activate",
		            GTK_SIGNAL_FUNC(set_data_type),
			    (gpointer)"random");
  gtk_signal_connect_object(GTK_OBJECT(sorted_item), "activate",
		            GTK_SIGNAL_FUNC(set_data_type),
			    (gpointer)"sorted");
  gtk_signal_connect_object(GTK_OBJECT(reverse_item), "activate",
		            GTK_SIGNAL_FUNC(set_data_type),
			    (gpointer)"reverse");
  gtk_signal_connect_object(GTK_OBJECT(a_sorted_item), "activate",
		            GTK_SIGNAL_FUNC(set_data_type),
			    (gpointer)"a_sorted");
  gtk_signal_connect_object(GTK_OBJECT(a_reverse_item), "activate",
		            GTK_SIGNAL_FUNC(set_data_type),
			    (gpointer)"a_reverse");

  gtk_widget_show(menu);

  gtk_option_menu_set_menu(GTK_OPTION_MENU(data_option_menu), menu);
  gtk_box_pack_start(GTK_BOX(toolbar), data_option_menu, FALSE, FALSE, 0);
  gtk_widget_show(data_option_menu);

  return(toolbar);

}

void set_sort_type(gchar *string)
{
  int t = 0;

  if (strcmp(string, "bubble") == 0){
    t = BUBBLE_SORT;
  }
  else if (strcmp(string, "insertion") == 0){
    t = INSERTION_SORT;
  }
  else if (strcmp(string, "selection") == 0){
    t = SELECTION_SORT;
  }
  else if (strcmp(string, "quick") == 0){
    t = QUICK_SORT;
  }

  if (t == sort_type) return;

  sort_type = t;

#ifdef DEBUG
  g_print("Sort type set to ");
  g_print(string);
  g_print("\n");
#endif

}
void set_data_type(gchar *string)
{
  int t = 0;

  if (strcmp(string, "sorted") == 0){
    t = SORTED;
  }
  else if (strcmp(string, "reverse") == 0){
    t = REVERSE;
  }
  else if (strcmp(string, "random") == 0){
    t = RANDOM;
  }
  else if (strcmp(string, "a_sorted") == 0){
    t = ALMOST_SORTED;
  }
  else if (strcmp(string, "a_reverse") == 0){
    t = ALMOST_REVERSE;
  }

  if (t == list_type) return;

  list_type = t;
  list_used = TRUE;
  reset_list(NULL, NULL);

#ifdef DEBUG
  g_print("List type set to ");
  g_print(string);
  g_print("\n");
#endif

}

